@@ -24,10 +24,6 @@
 
 #include <cf-agent-enterprise-stubs.h>
 
-ENTERPRISE_VOID_FUNC_2ARG_DEFINE_STUB(void, LastSawBundle, ARG_UNUSED const Bundle *, bundle, ARG_UNUSED double, comp)
-{
-}
-
 ENTERPRISE_FUNC_8ARG_DEFINE_STUB(PromiseResult, LogFileChange,
                                  ARG_UNUSED EvalContext *, ctx,
                                  ARG_UNUSED const char *, file,
@@ -42,23 +38,13 @@ ENTERPRISE_FUNC_8ARG_DEFINE_STUB(PromiseResult, LogFileChange,
     return PROMISE_RESULT_NOOP;
 }
 
-ENTERPRISE_VOID_FUNC_2ARG_DEFINE_STUB(void, Nova_NoteVarUsageDB,
-                                      ARG_UNUSED EvalContext *, ctx,
-                                      ARG_UNUSED const GenericAgentConfig *, config)
-{
-}
-
-ENTERPRISE_VOID_FUNC_2ARG_DEFINE_STUB(void, Nova_NoteClassUsage,
-                                      ARG_UNUSED EvalContext *, ctx,
-                                      ARG_UNUSED const GenericAgentConfig *, config)
-{
-}
-
 ENTERPRISE_VOID_FUNC_1ARG_DEFINE_STUB(void, Nova_TrackExecution, ARG_UNUSED const char *, input_file)
 {
 }
 
-ENTERPRISE_VOID_FUNC_1ARG_DEFINE_STUB(void, GenerateDiffReports, ARG_UNUSED const GenericAgentConfig *, config)
+ENTERPRISE_VOID_FUNC_2ARG_DEFINE_STUB(void, GenerateReports, 
+                                      ARG_UNUSED const GenericAgentConfig *, config, 
+                                      ARG_UNUSED const EvalContext *, ctx)
 {
 }
 
