@@ -58,6 +58,7 @@ static const char *const VMOUNTCOMM[] =
     [PLATFORM_CONTEXT_HP] = "/sbin/mount -ea",             /* hpux */
     [PLATFORM_CONTEXT_AIX] = "/usr/sbin/mount -t nfs",     /* aix */
     [PLATFORM_CONTEXT_LINUX] = "/bin/mount -va",           /* linux */
+    [PLATFORM_CONTEXT_BUSYBOX] = "/bin/mount -va",         /* linux */
     [PLATFORM_CONTEXT_SOLARIS] = "/usr/sbin/mount -a",     /* solaris */
     [PLATFORM_CONTEXT_SUN_SOLARIS] = "/usr/sbin/mount -a", /* solaris */
     [PLATFORM_CONTEXT_FREEBSD] = "/sbin/mount -va",        /* freebsd */
@@ -82,6 +83,7 @@ static const char *const VUNMOUNTCOMM[] =
     [PLATFORM_CONTEXT_HP] = "/sbin/umount",             /* hpux */
     [PLATFORM_CONTEXT_AIX] = "/usr/sbin/umount",        /* aix */
     [PLATFORM_CONTEXT_LINUX] = "/bin/umount",           /* linux */
+    [PLATFORM_CONTEXT_BUSYBOX] = "/bin/umount",         /* linux */
     [PLATFORM_CONTEXT_SOLARIS] = "/etc/umount",         /* solaris */
     [PLATFORM_CONTEXT_SUN_SOLARIS] = "/etc/umount",     /* solaris */
     [PLATFORM_CONTEXT_FREEBSD] = "/sbin/umount",        /* freebsd */
@@ -106,6 +108,7 @@ static const char *const VMOUNTOPTS[] =
     [PLATFORM_CONTEXT_HP] = "bg,hard,intr",          /* hpux */
     [PLATFORM_CONTEXT_AIX] = "bg,hard,intr",         /* aix */
     [PLATFORM_CONTEXT_LINUX] = "defaults",           /* linux */
+    [PLATFORM_CONTEXT_BUSYBOX] = "defaults",         /* linux */
     [PLATFORM_CONTEXT_SOLARIS] = "bg,hard,intr",     /* solaris */
     [PLATFORM_CONTEXT_SUN_SOLARIS] = "bg,hard,intr", /* solaris */
     [PLATFORM_CONTEXT_FREEBSD] = "bg,intr",          /* freebsd */
