@@ -49,6 +49,7 @@ const char *remove_keys_host = NULL;                            /* GLOBAL_A */
 static char *print_digest_arg = NULL;                           /* GLOBAL_A */
 static char *trust_key_arg = NULL;                              /* GLOBAL_A */
 static char *KEY_PATH = NULL;                                   /* GLOBAL_A */
+bool LOOKUP_HOSTS = true;                                       /* GLOBAL_A */
 
 static GenericAgentConfig *CheckOpts(int argc, char **argv);
 
@@ -78,6 +79,7 @@ static const struct option OPTIONS[] =
     {"trust-key", required_argument, 0, 't'},
     {"color", optional_argument, 0, 'C'},
     {"timestamp", no_argument, 0, TIMESTAMP_VAL},
+    {"numeric", no_argument, 0, 'n'},
     {NULL, 0, 0, '\0'}
 };
 
@@ -96,6 +98,7 @@ static const char *const HINTS[] =
     "Make cf-serverd/cf-agent trust the specified public key. Argument value is of the form [[USER@]IPADDR:]FILENAME where FILENAME is the local path of the public key for client at IPADDR address.",
     "Enable colorized output. Possible values: 'always', 'auto', 'never'. If option is used, the default value is 'auto'",
     "Log timestamps on each line of log output",
+    "Do not lookup host names",
     NULL
 };
 
@@ -111,7 +114,7 @@ static void handleShowKeysSignal(int signum)
     signal(signum, handleShowKeysSignal);
 }
 
-static void ThisAgentInit(CfKeySigHandler sighandler)
+static void SetupSignalsForCfKey(CfKeySigHandler sighandler)
 {
     signal(SIGINT, sighandler);
     signal(SIGTERM, sighandler);
@@ -123,6 +126,8 @@ static void ThisAgentInit(CfKeySigHandler sighandler)
 
 int main(int argc, char *argv[])
 {
+    SetupSignalsForCfKey(HandleSignalsForAgent);
+
     GenericAgentConfig *config = CheckOpts(argc, argv);
     EvalContext *ctx = EvalContextNew();
     GenericAgentConfigApply(ctx, config);
@@ -131,7 +136,7 @@ int main(int argc, char *argv[])
 
     if (SHOWHOSTS)
     {
-        ThisAgentInit(handleShowKeysSignal);
+        SetupSignalsForCfKey(handleShowKeysSignal);
         ShowLastSeenHosts();
         return 0;
     }
@@ -142,7 +147,6 @@ int main(int argc, char *argv[])
     }
 
     GenericAgentPostLoadInit(ctx);
-    ThisAgentInit(HandleSignalsForAgent);
 
     if (REMOVEKEYS)
     {
@@ -232,7 +236,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
     int c;
     GenericAgentConfig *config = GenericAgentConfigNewDefault(AGENT_TYPE_KEYGEN, GetTTYInteractive());
 
-    while ((c = getopt_long(argc, argv, "dvf:VMp:sr:xt:hl:C::",
+    while ((c = getopt_long(argc, argv, "dvf:VMp:sr:xt:hl:C::n",
                             OPTIONS, NULL))
            != -1)
     {
@@ -315,6 +319,10 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             LoggingEnableTimestamps(true);
             break;
 
+        case 'n':
+            LOOKUP_HOSTS = false;
+            break;
+
         default:
             {
                 Writer *w = FileWriter(stdout);
