@@ -21,6 +21,10 @@
   (COSL) may apply to this file if you as a licensee so wish it. See
   included file COSL.txt.
 */
+
+
+#include <platform.h>
+
 #include "server_access.h"
 #include "strlist.h"
 #include "server.h"
@@ -29,17 +33,30 @@
 #include <string_lib.h>                      /* StringMatchFull TODO REMOVE */
 #include <misc_lib.h>
 #include <file_lib.h>
+#include <regex.h>
+
 
 struct acl *paths_acl;
-struct acl *classes_acl, *vars_acl, *literals_acl;
+struct acl *classes_acl;
+struct acl *vars_acl;
+struct acl *literals_acl;
 struct acl *query_acl;
+struct acl *bundles_acl;
+struct acl *roles_acl;
+
 
 /**
  * Run this function on every resource (file, class, var etc) access to
  * grant/deny rights. Currently it checks if:
  *  1. #ipaddr matches the subnet expression in {admit,deny}_ips
  *  2. #hostname matches the subdomain expression in {admit,deny}_hostnames
- *  3. #key is found exactly as it is in {admit,deny}_keys
+ *  3. #key is searched as-is in {admit,deny}_keys
+ *  4. #username is searched as-is in {admit,deny}_usernames
+ *
+ * @param #found If not NULL then it returns whether the denial was implicit
+ *               (found=false) or explicit (found=true). If not NULL it also
+ *               changes the way the ACLs are traversed to a SLOWER mode,
+ *               since every entry has to be checked for explicit denial.
  *
  * @return Default is false, i.e. deny. If a match is found in #acl->admit.*
  *         then return true, unless a match is also found in #acl->deny.* in
@@ -51,20 +68,18 @@ struct acl *query_acl;
  *       lists, and running through these two lists on every file request will
  *       be much faster.
  */
-bool access_CheckResource(const struct resource_acl *acl,
-                          const char *ipaddr, const char *hostname,
-                          const char *key)
+static bool access_CheckResource(const struct resource_acl *acl,
+                                 bool *found,
+                                 const char *ipaddr, const char *hostname,
+                                 const char *key, const char *username)
 {
-    /* Only hostname may be NULL in case of resolution failure. */
-    assert(ipaddr != NULL);
-    assert(key != NULL);
-
-    bool access = false;                                 /* DENY by default */
+    bool access     = false;                 /* DENY by default */
+    bool have_match = false;                 /* No matching rule found yet */
 
     /* First we check for admission, secondly for denial, so that denial takes
      * precedence. */
 
-    if (acl->admit.ips)
+    if (!NULL_OR_EMPTY(ipaddr) && acl->admit.ips != NULL)
     {
         /* Still using legacy code here, doing linear search over all IPs in
          * textual representation... too CPU intensive! TODO store the ACL as
@@ -90,10 +105,11 @@ bool access_CheckResource(const struct resource_acl *acl,
                 "Admit IP due to rule: %s",
                 rule);
             access = true;
+            have_match = true;
         }
     }
-    if (!access && acl->admit.hostnames != NULL &&
-        hostname != NULL && *hostname != '\0')
+    if (!access && !NULL_OR_EMPTY(hostname) &&
+        acl->admit.hostnames != NULL)
     {
         size_t pos = StrList_SearchLongestPrefix(acl->admit.hostnames,
                                                  hostname, 0,
@@ -120,9 +136,11 @@ bool access_CheckResource(const struct resource_acl *acl,
                 "Admit hostname due to rule: %s",
                 StrList_At(acl->admit.hostnames, pos));
             access = true;
+            have_match = true;
         }
     }
-    if (!access && acl->admit.keys != NULL)
+    if (!access && !NULL_OR_EMPTY(key) &&
+        acl->admit.keys != NULL)
     {
         size_t pos;
         bool ret = StrList_BinarySearch(acl->admit.keys, key, &pos);
@@ -132,13 +150,44 @@ bool access_CheckResource(const struct resource_acl *acl,
                 "Admit key due to rule: %s",
                 StrList_At(acl->admit.keys, pos));
             access = true;
+            have_match = true;
+        }
+    }
+    if (!access && !NULL_OR_EMPTY(username) &&
+        acl->admit.usernames != NULL)
+    {
+        size_t pos;
+        bool ret = StrList_BinarySearch(acl->admit.usernames, username, &pos);
+        if (ret)
+        {
+            Log(LOG_LEVEL_DEBUG,
+                "Admit username due to rule: %s",
+                StrList_At(acl->admit.usernames, pos));
+            access = true;
+            have_match = true;
         }
     }
 
 
-    /* If access has been granted, we might need to deny it based on ACL. */
+    /* An admit rule was not found, and we don't care whether the denial is
+     * explicit or implicit: we can finish now. */
+    if (!access && found == NULL)
+    {
+        assert(!have_match);
+        return false;                                      /* EARLY RETURN! */
+    }
 
-    if (access && acl->deny.ips != NULL)
+    /* If access has been granted, we might need to deny it based on ACL. */
+    /* Same goes if access has not been granted and "found" is not NULL, in
+     * which case we have to return in "found", whether an explicit denial
+     * rule matched or not. */
+
+    assert((access && have_match) ||
+           (!access && !have_match && found != NULL));
+
+    if ((access || !have_match) &&
+        !NULL_OR_EMPTY(ipaddr) &&
+        acl->deny.ips != NULL)
     {
         const char *rule = NULL;
         for (int i = 0; i < StrList_Len(acl->deny.ips); i++)
@@ -158,10 +207,12 @@ bool access_CheckResource(const struct resource_acl *acl,
                 "Deny IP due to rule: %s",
                 rule);
             access = false;
+            have_match = true;
         }
     }
-    if (access && acl->deny.hostnames != NULL &&
-        hostname != NULL && *hostname != '\0')
+    if ((access || !have_match) &&
+        !NULL_OR_EMPTY(hostname) &&
+        acl->deny.hostnames != NULL)
     {
         size_t pos = StrList_SearchLongestPrefix(acl->deny.hostnames,
                                                  hostname, 0,
@@ -188,9 +239,12 @@ bool access_CheckResource(const struct resource_acl *acl,
                 "Deny hostname due to rule: %s",
                 StrList_At(acl->deny.hostnames, pos));
             access = false;
+            have_match = true;
         }
     }
-    if (access && acl->deny.keys != NULL)
+    if ((access || !have_match) &&
+        !NULL_OR_EMPTY(key) &&
+        acl->deny.keys != NULL)
     {
         size_t pos;
         bool ret = StrList_BinarySearch(acl->deny.keys, key, &pos);
@@ -200,12 +254,37 @@ bool access_CheckResource(const struct resource_acl *acl,
                 "Deny key due to rule: %s",
                 StrList_At(acl->deny.keys, pos));
             access = false;
+            have_match = true;
+        }
+    }
+    if ((access || !have_match) &&
+        !NULL_OR_EMPTY(username) &&
+        acl->deny.usernames != NULL)
+    {
+        size_t pos;
+        bool ret = StrList_BinarySearch(acl->deny.usernames, username, &pos);
+        if (ret)
+        {
+            Log(LOG_LEVEL_DEBUG,
+                "Deny username due to rule: %s",
+                StrList_At(acl->deny.usernames, pos));
+            access = false;
+            have_match = true;
         }
     }
 
+    /* We can't have implicit admittance,
+       admittance must always be explicit. */
+    assert(! (access && !have_match));
+
+    if (found != NULL)
+    {
+        *found = have_match;
+    }
     return access;
 }
 
+
 /**
  * Search #req_path in #acl, if found check its rules. The longest parent
  * directory of #req_path is searched, or an exact match. Directories *must*
@@ -229,7 +308,7 @@ bool acl_CheckPath(const struct acl *acl, const char *reqpath,
     if (pos != (size_t) -1)                          /* acl entry was found */
     {
         const struct resource_acl *racl = &acl->acls[pos];
-        bool ret = access_CheckResource(racl, ipaddr, hostname, key);
+        bool ret = access_CheckResource(racl, NULL, ipaddr, hostname, key, NULL);
         if (ret == true)                  /* entry found that grants access */
         {
             access = true;
@@ -265,9 +344,10 @@ bool acl_CheckPath(const struct acl *acl, const char *reqpath,
             const struct resource_acl *racl = &acl->acls[pos2];
             /* Check if the magic strings are allowed or denied. */
             bool ret =
-                access_CheckResource(racl, "$(connection.ip)",
+                access_CheckResource(racl, NULL,
+                                     "$(connection.ip)",
                                      "$(connection.hostname)",
-                                     "$(connection.key)");
+                                     "$(connection.key)", NULL);
             if (ret == true)                  /* entry found that grants access */
             {
                 access = true;
@@ -293,7 +373,8 @@ bool acl_CheckExact(const struct acl *acl, const char *req_string,
     if (found)
     {
         const struct resource_acl *racl = &acl->acls[pos];
-        bool ret = access_CheckResource(racl, ipaddr, hostname, key);
+        bool ret = access_CheckResource(racl, NULL,
+                                        ipaddr, hostname, key, NULL);
         if (ret == true)                  /* entry found that grants access */
         {
             access = true;
@@ -303,6 +384,54 @@ bool acl_CheckExact(const struct acl *acl, const char *req_string,
     return access;
 }
 
+/**
+ * Go linearly over all the #acl and check every rule if it matches.
+ * ADMIT only if at least one rule matches admit and none matches deny.
+ * DENY if no rule matches OR if at least one matches deny.
+ */
+bool acl_CheckRegex(const struct acl *acl, const char *req_string,
+                    const char *ipaddr, const char *hostname,
+                    const char *key, const char *username)
+{
+    bool retval = false;
+
+    /* For all ACLs */
+    for (size_t i = 0; i < acl->len; i++)
+    {
+        const char *regex = acl->resource_names->list[i]->str;
+
+        /* Does this ACL matches the req_string? */
+        if (StringMatchFull(regex, req_string))
+        {
+            const struct resource_acl *racl = &acl->acls[i];
+
+            /* Does this ACL apply to this host? */
+            bool found;
+            bool admit = access_CheckResource(racl, &found,
+                                              ipaddr, hostname, key, username);
+            if (found && !admit)
+            {
+                return false;
+            }
+            else if (found && admit)
+            {
+                retval = true;
+            }
+            else
+            {
+                /* If it's not found, there should be no admittance. */
+                assert(!found);
+                assert(!admit);
+                /* We are not touching retval, because it was possibly found
+                 * before and retval has been set to "true". */
+            }
+        }
+    }
+
+    return retval;
+}
+
+
 /**
  * Search the list of resources for the handle. If found return the index of
  * the resource ACL that corresponds to that handle, else add the handle with
@@ -376,6 +505,8 @@ size_t acl_SortedInsert(struct acl **a, const char *handle)
     Log(LOG_LEVEL_DEBUG, "Inserted in ACL position %zu: %s",
         position, handle);
 
+    assert(acl->len == StrList_Len(acl->resource_names));
+
     return position;
 }
 
@@ -396,3 +527,48 @@ void acl_Free(struct acl *a)
 
     free(a);
 }
+
+void acl_Summarise(const struct acl *acl, const char *title)
+{
+    assert(acl->len == StrList_Len(acl->resource_names));
+
+    size_t i, j;
+    for (i = 0; i < acl->len; i++)
+    {
+        Log(LOG_LEVEL_VERBOSE, "\t%s: %s",
+            title, StrList_At(acl->resource_names, i));
+
+        const struct resource_acl *racl = &acl->acls[i];
+
+        for (j = 0; j < StrList_Len(racl->admit.ips); j++)
+        {
+            Log(LOG_LEVEL_VERBOSE, "\t\tadmit_ips: %s",
+                StrList_At(racl->admit.ips, j));
+        }
+        for (j = 0; j < StrList_Len(racl->admit.hostnames); j++)
+        {
+            Log(LOG_LEVEL_VERBOSE, "\t\tadmit_hostnames: %s",
+                StrList_At(racl->admit.hostnames, j));
+        }
+        for (j = 0; j < StrList_Len(racl->admit.keys); j++)
+        {
+            Log(LOG_LEVEL_VERBOSE, "\t\tadmit_keys: %s",
+                StrList_At(racl->admit.keys, j));
+        }
+        for (j = 0; j < StrList_Len(racl->deny.ips); j++)
+        {
+            Log(LOG_LEVEL_VERBOSE, "\t\tdeny_ips: %s",
+                StrList_At(racl->deny.ips, j));
+        }
+        for (j = 0; j < StrList_Len(racl->deny.hostnames); j++)
+        {
+            Log(LOG_LEVEL_VERBOSE, "\t\tdeny_hostnames: %s",
+                StrList_At(racl->deny.hostnames, j));
+        }
+        for (j = 0; j < StrList_Len(racl->deny.keys); j++)
+        {
+            Log(LOG_LEVEL_VERBOSE, "\t\tdeny_keys: %s",
+                StrList_At(racl->deny.keys, j));
+        }
+    }
+}
