@@ -116,6 +116,18 @@ int RecvSocketStream(int sd, char buffer[CF_BUFSIZE], int toget)
 
 /*************************************************************************/
 
+/**
+ * @brief Send #tosend bytes from buffer to sd.
+ * @param sd Socket descriptor
+ * @param buffer Buffer from which to read data
+ * @param tosend Number of bytes to write.
+ *
+ * @return number of bytes actually sent, must be equal to #tosend
+ *         -1  in case of timeout or error - socket is unusable
+ *         -1  also in case of early proper connection close
+ *         0        NEVER
+ *         <tosend  NEVER
+ */
 int SendSocketStream(int sd, const char buffer[CF_BUFSIZE], int tosend)
 {
     int sent, already = 0;
@@ -140,12 +152,13 @@ int SendSocketStream(int sd, const char buffer[CF_BUFSIZE], int tosend)
 
         if (sent == -1)
         {
-            Log(LOG_LEVEL_VERBOSE, "Couldn't send. (send: %s)", GetErrorStr());
+            Log(LOG_LEVEL_ERR, "Couldn't send. (send: %s)", GetErrorStr());
             return -1;
         }
 
         already += sent;
     } while (already < tosend);
+    assert(already == tosend);
 
     return already;
 }
