@@ -38,16 +38,19 @@ extern char BINDINTERFACE[];                  /* cf3globals.c, cf3.extern.h */
 
 /**
  * @param len is the number of bytes to send, or 0 if buffer is a
- *        '\0'-terminated string so strlen(buffer) can used.
+ *        '\0'-terminated string so strlen(buffer) can be used.
  * @return -1 in case of error or connection closed
  *         (also currently returns 0 for success but don't count on it)
  * @NOTE #buffer can't be of zero length, our protocol
- *       does not allow empty transactions! The reason is that
- *       ReceiveTransaction() can't differentiate between that
- *       and connection closed.
+ *       does not allow empty transactions!
  * @NOTE (len <= CF_BUFSIZE - CF_INBAND_OFFSET)
+ *
+ * @TODO Currently only transactions up to CF_BUFSIZE-CF_INBAND_OFFSET are
+ *       allowed to be sent. This function should be changed to allow up to
+ *       CF_BUFSIZE-1 (since '\0' is not sent, but the receiver needs space to
+ *       append it). So transaction length will be at most 4095!
  */
-int SendTransaction(const ConnectionInfo *conn_info,
+int SendTransaction(ConnectionInfo *conn_info,
                     const char *buffer, int len, char status)
 {
     assert(status == CF_MORE || status == CF_DONE);
@@ -60,8 +63,7 @@ int SendTransaction(const ConnectionInfo *conn_info,
         len = strlen(buffer);
     }
 
-    /* Not allowed to send zero-payload packets, because
-       (ReceiveTransaction() == 0) currently means connection closed. */
+    /* Not allowed to send zero-payload packets */
     assert(len > 0);
 
     if (len > CF_BUFSIZE - CF_INBAND_OFFSET)
@@ -103,7 +105,14 @@ int SendTransaction(const ConnectionInfo *conn_info,
 
     if (ret == -1)
     {
-        return -1;                                              /* error */
+        /* We are experiencing problems with sending data to server.
+         * This might lead to packages being not delivered in correct
+         * order and unexpected issues like directories being replaced
+         * with files.
+         * In order to make sure that file transfer is reliable we have to
+         * close connection to avoid broken packages being received. */
+        conn_info->status = CONNECTIONINFO_STATUS_BROKEN;
+        return -1;
     }
     else
     {
@@ -120,8 +129,10 @@ int SendTransaction(const ConnectionInfo *conn_info,
  *  Receive a transaction packet of at most CF_BUFSIZE-1 bytes, and
  *  NULL-terminate it.
  *
- *  @return 0 in case of socket closed, -1 in case of other error or timeout.
- *              In both cases the connection MAY NOT BE FINALISED!
+ *  @param #buffer must be of size at least CF_BUFSIZE.
+ *
+ *  @return -1 in case of closed socket, other error or timeout.
+ *              The connection MAY NOT BE FINALISED!
  *          >0 the number of bytes read, transaction was successfully received.
  *
  *  @TODO shutdown() the connection in all cases were this function returns -1,
@@ -149,7 +160,7 @@ int ReceiveTransaction(ConnectionInfo *conn_info, char *buffer, int *more)
 
     /* If error occured or recv() timeout or if connection was gracefully
      * closed. Connection has been finalised. */
-    if (ret == -1 || ret == 0)
+    if (ret <= 0)
     {
         /* We are experiencing problems with receiving data from server.
          * This might lead to packages being not delivered in correct
@@ -158,7 +169,7 @@ int ReceiveTransaction(ConnectionInfo *conn_info, char *buffer, int *more)
          * In order to make sure that file transfer is reliable we have to
          * close connection to avoid broken packages being received. */
         conn_info->status = CONNECTIONINFO_STATUS_BROKEN;
-        return ret;
+        return -1;
     }
     else if (ret != CF_INBAND_OFFSET)
     {
@@ -240,12 +251,9 @@ int ReceiveTransaction(ConnectionInfo *conn_info, char *buffer, int *more)
         ret = -1;
     }
 
-    if (ret <= 0)
-    {
-        conn_info->status = CONNECTIONINFO_STATUS_BROKEN;
-        return ret;
-    }
-    else if (ret != len)
+    /* Connection gracefully closed (ret==0) or connection error (ret==-1) or
+     * just partial receive of bytestream.*/
+    if (ret != len)
     {
         /*
          * Should never happen except with TLS, given that we are using
@@ -501,14 +509,14 @@ int SocketConnect(const char *host, const char *port,
     if (connected)
     {
         Log(LOG_LEVEL_VERBOSE,
-            "Connected to host %s address %s port %s",
-            host, txtaddr, port);
+            "Connected to host %s address %s port %s (socket descriptor %d)",
+            host, txtaddr, port, sd);
     }
     else
     {
         Log(LOG_LEVEL_VERBOSE,
-            "Unable to connect to host %s port %s",
-            host, port);
+            "Unable to connect to host %s port %s (socket descriptor %d)",
+            host, port, sd);
     }
 
     return sd;
@@ -545,12 +553,14 @@ int SocketConnect(const char *host, const char *port,
 bool TryConnect(int sd, unsigned long timeout_ms,
                 const struct sockaddr *sa, socklen_t sa_len)
 {
+    assert(sd != -1);
     assert(sa != NULL);
 
     if (sd >= FD_SETSIZE)
     {
-        Log(LOG_LEVEL_ERR, "Open connections exceed FD_SETSIZE limit of %d",
-            FD_SETSIZE);
+        Log(LOG_LEVEL_ERR,
+            "Open connections exceed FD_SETSIZE limit (%d >= %d)",
+            sd, FD_SETSIZE);
         return false;
     }
 
