@@ -34,7 +34,7 @@
 extern uint32_t bwlimit_kbytes;
 
 
-int SendTransaction(const ConnectionInfo *conn_info, const char *buffer, int len, char status);
+int SendTransaction(ConnectionInfo *conn_info, const char *buffer, int len, char status);
 int ReceiveTransaction(ConnectionInfo *conn_info, char *buffer, int *more);
 
 int SetReceiveTimeout(int fd, unsigned long ms);
