@@ -41,11 +41,11 @@ bool TLSGenericInitialize(void);
 int TLSVerifyCallback(X509_STORE_CTX *ctx, void *arg);
 int TLSVerifyPeer(ConnectionInfo *conn_info, const char *remoteip, const char *username);
 X509 *TLSGenerateCertFromPrivKey(RSA *privkey);
-void TLSLogError(SSL *ssl, LogLevel level, const char *prepend, int code);
+int TLSLogError(SSL *ssl, LogLevel level, const char *prepend, int code);
 int TLSSend(SSL *ssl, const char *buffer, int length);
-int TLSRecv(SSL *ssl, char *buffer, int length);
+int TLSRecv(SSL *ssl, char *buffer, int toget);
 int TLSRecvLines(SSL *ssl, char *buf, size_t buf_size);
-void TLSSetDefaultOptions(SSL_CTX *ssl_ctx);
+void TLSSetDefaultOptions(SSL_CTX *ssl_ctx, const char *min_version);
 const char *TLSErrorString(intmax_t errcode);
 
 #endif
