@@ -25,7 +25,8 @@
 #ifndef CFENGINE_SYSINFO_H
 #define CFENGINE_SYSINFO_H
 
-#include <cf3.defs.h>
+/* TODO libpromises depends on libenv, the opposite should not happen! */
+#include <eval_context.h>
 
 void DetectEnvironment(EvalContext *ctx);
 
@@ -33,4 +34,8 @@ void CreateHardClassesFromCanonification(EvalContext *ctx, const char *canonifie
 int GetUptimeMinutes(time_t now);
 int GetUptimeSeconds(time_t now);
 
+void GetInterfacesInfo(EvalContext *ctx);
+void GetNetworkingInfo(EvalContext *ctx);
+JsonElement* GetNetworkingConnections(EvalContext *ctx);
+
 #endif
