@@ -161,6 +161,10 @@ static bool OpenTokyoDatabase(const char *filename, TCHDB **hdb)
     return true;
 }
 
+void DBPrivSetMaximumConcurrentTransactions(ARG_UNUSED int max_txn)
+{
+}
+
 DBPriv *DBPrivOpenDB(const char *dbpath, ARG_UNUSED dbid id)
 {
     DBPriv *db = xcalloc(1, sizeof(DBPriv));
@@ -217,9 +221,33 @@ void DBPrivCommit(ARG_UNUSED DBPriv *db)
 {
 }
 
+bool DBPrivClean(DBPriv *db)
+{
+    DBCursorPriv *cursor = DBPrivOpenCursor(db);
+    
+    if (!cursor)
+    {
+        return false;
+    }
+    
+    void *key;
+    int key_size;
+    void *value;
+    int value_size;
+    
+    while ((DBPrivAdvanceCursor(cursor, &key, &key_size, &value, &value_size)))
+    {
+        DBPrivDeleteCursorEntry(cursor);
+    }
+    
+    DBPrivCloseCursor(cursor);
+    
+    return true;
+}
+
 bool DBPrivHasKey(DBPriv *db, const void *key, int key_size)
 {
-    // FIXME: distinguish between "entry not found" and "error occured"
+    // FIXME: distinguish between "entry not found" and "error occurred"
 
     return tchdbvsiz(db->hdb, key, key_size) != -1;
 }
@@ -399,15 +427,15 @@ char *DBPrivDiagnose(const char *dbpath)
         return StringFormat("Error seeking to end: %s\n", strerror(errno));
     }
 
-    uint64_t size = ftell(fp);
+    long size = ftell(fp);
     if(size < 256)
     {
         fclose(fp);
-        return StringFormat("Seek-to-end size less than minimum required: %zd", size);
+        return StringFormat("Seek-to-end size less than minimum required: %ld", size);
     }
 
     char hbuf[256];
-    memset(hbuf, 0, (size_t)256);
+    memset(hbuf, 0, sizeof(hbuf));
 
     if(fseek(fp, 0, SEEK_SET) != 0)
     {
@@ -428,21 +456,22 @@ char *DBPrivDiagnose(const char *dbpath)
     }
 
     uint64_t declared_size = 0;
-    memcpy(&declared_size, hbuf+56, sizeof(uint64_t));
-    if (declared_size == size)
+    /* Read file size from tchdb header. It is stored in little endian. */
+    memcpy(&declared_size, &hbuf[56], sizeof(uint64_t));
+    if (declared_size == (uint64_t) size)
     {
         return NULL; // all is well
     }
     else
     {
         declared_size = SWAB64(declared_size);
-        if (declared_size == size)
+        if (declared_size == (uint64_t) size)
         {
-            return StringFormat("Endianness mismatch, declared size SWAB64 '%zd' equals seek-to-end size '%zd'", declared_size, size);
+            return StringFormat("Endianness mismatch, declared size SWAB64 '%ju' equals seek-to-end size '%ld'", (uintmax_t) declared_size, size);
         }
         else
         {
-            return StringFormat("Size mismatch, declared size SWAB64 '%zd', seek-to-end-size '%zd'", declared_size, size);
+            return StringFormat("Size mismatch, declared size SWAB64 '%ju', seek-to-end-size '%ld'", (uintmax_t) declared_size, size);
         }
     }
 }
