@@ -30,7 +30,7 @@
 
 int IsExecutable(const char *file);
 bool ShellCommandReturnsZero(const char *command, ShellType shell);
-bool GetExecOutput(const char *command, char *buffer, ShellType shell);
+bool GetExecOutput(const char *command, char **buffer, size_t *buffer_size, ShellType shell);
 void ActAsDaemon();
 char **ArgSplitCommand(const char *comm);
 void ArgFree(char **args);
