@@ -49,11 +49,10 @@ const char *FirstFileSeparator(const char *str);
 const char *LastFileSeparator(const char *str);
 bool ChopLastNode(char *str);
 char *CanonifyName(const char *str);
-void CanonifyNameInPlace(char *str);
 void TransformNameInPlace(char *s, char from, char to);
 char *CanonifyChar(const char *str, char ch);
 const char *ReadLastNode(const char *str);
-int CompressPath(char *dest, const char *src);
+bool CompressPath(char *dest, size_t dest_size, const char *src);
 bool IsFileOutsideDefaultRepository(const char *f);
 int RootDirLength(const char *f);
 const char *GetSoftwareCacheFilename(char *buffer);
