@@ -89,7 +89,7 @@ const char *GetDefault##FUNC##Dir(void)                             \
 }                                                                   \
 
 GET_DEFAULT_DIRECTORY_DEFINE(Work, work, WORKDIR, NULL)
-GET_DEFAULT_DIRECTORY_DEFINE(Log, log, LOGDIR, NULL)
+GET_DEFAULT_DIRECTORY_DEFINE(Log, log, LOGDIR, "log")
 GET_DEFAULT_DIRECTORY_DEFINE(Pid, pid, PIDDIR, NULL)
 GET_DEFAULT_DIRECTORY_DEFINE(Master, master, MASTERDIR, "masterfiles")
 GET_DEFAULT_DIRECTORY_DEFINE(Input, input, INPUTDIR, "inputs")
