@@ -25,10 +25,15 @@
 #ifndef CFENGINE_KNOWN_DIRS_H
 #define CFENGINE_KNOWN_DIRS_H
 
+#include <platform.h>
+
+const char *GetDefaultDir_helper(char dir[PATH_MAX], const char *root_dir, const char *append_dir);
+
 const char *GetWorkDir(void);
 const char *GetLogDir(void);
 const char *GetPidDir(void);
 const char *GetMasterDir(void);
 const char *GetInputDir(void);
+const char *GetStateDir(void);
 
 #endif
