@@ -35,7 +35,7 @@ typedef enum
     SERVER_CONTROL_ALLOW_USERS,
     SERVER_CONTROL_AUDITING,
     SERVER_CONTROL_BIND_TO_INTERFACE,
-    SERVER_CONTROL_CF_RUN_COMMAND,
+    SERVER_CONTROL_CFRUNCOMMAND,
     SERVER_CONTROL_CALL_COLLECT_INTERVAL,
     SERVER_CONTROL_CALL_COLLECT_WINDOW,
     SERVER_CONTROL_DENY_BAD_CLOCKS,
