@@ -37,8 +37,8 @@ typedef struct
 
 static bool GetProcessStat(pid_t pid, ProcessStat *state)
 {
-    char filename[CF_BUFSIZE];
-    snprintf(filename, CF_BUFSIZE, "/proc/%d/stat", (int)pid);
+    char filename[64];
+    xsnprintf(filename, sizeof(filename), "/proc/%jd/stat", (intmax_t) pid);
 
     int fd;
     for (;;)
@@ -143,12 +143,13 @@ ProcessState GetProcessState(pid_t pid)
     ProcessStat st;
     if (GetProcessStat(pid, &st))
     {
-        if (st.state == 'T')
+        switch (st.state)
         {
+        case 'T':
             return PROCESS_STATE_STOPPED;
-        }
-        else
-        {
+        case 'Z':
+            return PROCESS_STATE_ZOMBIE;
+        default:
             return PROCESS_STATE_RUNNING;
         }
     }
