@@ -1195,7 +1195,8 @@ static const char *FnCallCategoryToString(FnCallCategory category)
         [FNCALL_CATEGORY_FILES] = "files",
         [FNCALL_CATEGORY_IO] = "io",
         [FNCALL_CATEGORY_SYSTEM] = "system",
-        [FNCALL_CATEGORY_UTILS] = "utils"
+        [FNCALL_CATEGORY_UTILS] = "utils",
+        [FNCALL_CATEGORY_INTERNAL] = "internal"
     };
 
     return category_str[category];
@@ -1225,6 +1226,7 @@ static JsonElement *FnCallTypeToJson(const FnCallType *fn_syntax)
 
     JsonObjectAppendBool(json_fn, "variadic", fn_syntax->options & FNCALL_OPTION_VARARG);
     JsonObjectAppendBool(json_fn, "cached", fn_syntax->options & FNCALL_OPTION_CACHED);
+    JsonObjectAppendBool(json_fn, "collecting", fn_syntax->options & FNCALL_OPTION_COLLECTING);
     JsonObjectAppendString(json_fn, "category", FnCallCategoryToString(fn_syntax->category));
 
     return json_fn;
