@@ -30,15 +30,27 @@
 #include <string_expressions.h>
 #include <policy.h>
 
+/**
+   VarRef is immutable, which means that after allocated the members never
+   change, until all of it is freed.
+
+   @TODO constify all pointers returned from VarRef initializers (VarRefCopy,
+         VarRefParse, etc)
+*/
 typedef struct
 {
-    size_t hash;
-    size_t name_index_count;
     char *ns;
     char *scope;
     char *lval;
     char **indices;
     size_t num_indices;
+
+    /* TODO performance: when using VarRefCopy() we just need to allocate one
+     *      big chunk with malloc() and all pointers can point in there.  This
+     *      whole struct can be a single malloc'ed chunk with array[] space in
+     *      the end, if our use allows it (no changes after creation). */
+    /* bool single_alloc; */
+    /* char space[];      */
 } VarRef;
 
 VarRef *VarRefCopy(const VarRef *ref);
@@ -48,15 +60,17 @@ VarRef *VarRefCopyIndexless(const VarRef *ref);
 VarRef *VarRefParse(const char *var_ref_string);
 
 /**
- * @brief Parse the variable reference in the context of a bundle. This means that the VarRef will inherit scope and namespace
- *        of the bundle if these are not specified explicitly in the string.
+ * @brief Parse the variable reference in the context of a bundle. This means
+ *        that the VarRef will inherit scope and namespace of the bundle if
+ *        these are not specified explicitly in the string.
  */
 VarRef *VarRefParseFromBundle(const char *var_ref_string, const Bundle *bundle);
 VarRef *VarRefParseFromScope(const char *var_ref_string, const char *scope);
 VarRef *VarRefParseFromNamespaceAndScope(const char *qualified_name, const char *_ns, const char *_scope, char ns_separator, char scope_separator);
 VarRef VarRefConst(const char *ns, const char *scope, const char *lval);
 
-void VarRefDestroy(VarRef *ref);
+void VarRefDestroy        (VarRef *ref);
+void VarRefDestroy_untyped(void   *ref);
 
 char *VarRefToString(const VarRef *ref, bool qualified);
 
@@ -71,4 +85,11 @@ void VarRefAddIndex(VarRef *ref, const char *index);
 
 int VarRefCompare(const VarRef *a, const VarRef *b);
 
+bool VarRefEqual_untyped(const void *a, const void *b);
+
+unsigned int VarRefHash_untyped(const void *ref,
+                                unsigned int seed,
+                                unsigned int max);
+
+
 #endif
