@@ -33,6 +33,7 @@
 
 #include <sequence.h>                                           /* Seq */
 
+#define CFENGINE_REGEX_WHITESPACE_IN_CONTEXTS ".*[_A-Za-z0-9][ \\t]+[_A-Za-z0-9].*"
 
 /* Try to use CompileRegex() and StringMatchWithPrecompiledRegex(). */
 pcre *CompileRegex(const char *regex);
@@ -42,7 +43,7 @@ bool StringMatchWithPrecompiledRegex(pcre *regex, const char *str,
 bool StringMatchFull(const char *regex, const char *str);
 bool StringMatchFullWithPrecompiledRegex(pcre *regex, const char *str);
 Seq *StringMatchCaptures(const char *regex, const char *str, const bool return_names);
+Seq *StringMatchCapturesWithPrecompiledRegex(const pcre *pattern, const char *str, const bool return_names);
 bool CompareStringOrRegex(const char *value, const char *compareTo, bool regex);
 
-
 #endif  /* CFENGINE_REGEX_H */
