@@ -28,7 +28,8 @@
 #include <string_lib.h>
 #include <buffer.h>
 
-TYPED_SET_DEFINE(String, char *, (MapHashFn)&StringHash, (MapKeyEqualFn)&StringSafeEqual, &free)
+TYPED_SET_DEFINE(String, char *,
+                 StringHash_untyped, StringSafeEqual_untyped, free)
 
 Set *SetNew(MapHashFn element_hash_fn,
             MapKeyEqualFn element_equal_fn,
